function clu = clust_train(clu,model,batch,opt_evo)
%CLUST_TRAIN performs the training phase of the fds evolving clustering algorithm.
%   CLU = CLUST_TRAIN(CLU,MODEL,BATCH,OPT_EVO) add the parameter vector 
%   provided by MODEL arriving at BATCH in the cluster structure CLU,
%   assuming they are belonging to the nominal cluster. Options recorded 
%   in OPT_EVO are used. 
%
%   A single nominal cluster is supposed to appear in the nominal state
%
%   MODEL is generated with the function MODEL_CREATIION, while OPT_EVO is 
%   generated with PARAMETER_EVO
%
%   See also CLUST_EVOL, INIT_CLUST,

%   Copyright 2013 Francesco Trovo' Politecnico di Milano
%   $Revision: 1.0$  $Date: 2013/04/04$

for m = 1:length(model.eps_fau)

    clu.samples = [clu.samples; model.res_fau'];
    clu.train_samples = clu.samples;
    clu.raw_data.Y = [clu.raw_data.Y model.Y];
    clu.raw_data.Z = [clu.raw_data.Z model.Z];

    if isempty(clu.points_nor)
        clu.points_nor{1} = model.res_fau';
        clu.n_points_nor(1) = 1;
        clu.time_nor{1} = batch;        
        clu.n_clust_nor = 1;
        clu.added_point_nor{1} = [];
        clu.n_added_point_nor(1) = 0;
        clu.added_time_nor{1} = [];
    else
        clu.points_nor{1} = [clu.points_nor{1}; model.res_fau'];
        clu.n_points_nor(1) = clu.n_points_nor(1) + 1;
        clu.time_nor{1} = [clu.time_nor{1} batch];
    end
    clu.med_nor{1} = mean(clu.points_nor{1});
    clu.vari_nor{1} = update_cov(clu,1,'n',opt_evo.up_cov);
end

% Updating mean and variance
clu.mean = mean(clu.samples);
clu.variance = cov(clu.samples);
clu.n_points = sum(clu.n_points_nor);
clu.last_time_instant = batch;
