function cova = update_cov(cl,clust,clust_type,update_type)

% Input
%   cl: a clustering class
%   clust: cluster number
%   model: necessary for theoretical covariance estimation
%   clust_type: 'n' - nominal
%               'f' - faulty
%   update_type: 'g' - gaussian style
%                'l' - ljung style
%
% Output
%   cova: covariance matrix for the cluster points

if strcmp(clust_type,'n')
    if strcmp(update_type,'g')
        cova =  cov(cl.points_nor{clust});
    elseif strcmp(update_type,'l')
        indexes = cl.time_nor{clust};
        mu = cl.med_nor{clust};
        W_N = zeros(length(mu));
        U_N = zeros(length(mu));
        for i = indexes
            y = cl.raw_data.Y{i};
            z = cl.raw_data.Z{i};
            N = size(y,2);
            U = (mu * z - y) * z';
            U_N = U_N + 4 / N * (U' * U);
            W_N = W_N + 2 / N * (z * z');
        end
        W_N = W_N / length(indexes);
        U_N = U_N / length(indexes);
        cova = pinv(W_N) * U_N * pinv(W_N) / N;
        cova = (cova + cova') / 2; % avoid numerical problems
        [~,p] = chol(cova);
        if p ~= 0
           cova =  cov(cl.points_nor{clust});
        end
    end
elseif strcmp(clust_type,'f')
    if strcmp(update_type,'g')
        cova =  cov(cl.points_fau{clust});
    elseif strcmp(update_type,'l')
        indexes = cl.time_fau{clust};
        mu = cl.med_fau{clust};
        W_N = zeros(length(mu));
        U_N = zeros(length(mu));
        for i = indexes
            y = cl.raw_data.Y{i};
            z = cl.raw_data.Z{i};
            N = size(y,2);
            U = (mu * z - y) * z';
            U_N = U_N + 4 / N * (U' * U);
            W_N = W_N + 2 / N * (z * z');
        end
        W_N = W_N / length(indexes);
        U_N = U_N / length(indexes);
        cova = pinv(W_N) * U_N * pinv(W_N) / N;
        cova = (cova + cova') / 2; % avoid numerical problems
        [~,p] = chol(cova);
        if p ~= 0
           cova =  cov(cl.points_fau{clust});
        end
    end
end