function [sig pval] = granger(X,alpha,opt_gra)

% opt_gra.regmode   = 'OLS';  % VAR model estimation regression mode ('OLS', 'LWR' or empty for default)
% opt_gra.icregmode = 'LWR';  % information criteria regression mode ('OLS', 'LWR' or empty for default)
% 
% opt_gra.select_crit    = 'AIC';  % model order to use ('actual', 'AIC', 'BIC' or supplied numerical value)
% opt_gra.max_order     = 20;     % maximum model order for model order estimation
% 
% 
% opt_gra.tstat     = 'F';     % statistical test for MVGC:
%                              % 'F' for Granger's F-test (default)
%                              % 'chi2' for Geweke's chi2 test
% opt_gra.mhtc      = 'Bonferroni';  % multiple hypothesis test correction (see routine 'significance')
% 
% opt_gra.fs        = 1;      % sample rate (Hz)
% opt_gra.fres      = [];     % frequency resolution (empty for automatic calculation)
% 
% opt_gra.visual = 1;         % to visualize results
% 
% % opt_gra.madthreshold = 1e-5;

%% Model order estimation
[nvars nobs] = size(X);
assert(nvars < nobs);

% Calculate information criteria up to specified maximum model order.
[AIC,BIC,moAIC,moBIC] = tsdata_to_infocrit(X,opt_gra.max_order,opt_gra.icregmode,0);

if opt_gra.visual == 1
    figure(1); clf;
    plot_tsdata([AIC BIC]',{'AIC','BIC'},1/opt_gra.fs);
    title('Model order estimation');

    fprintf('\nbest model order (AIC) = %d\n',moAIC);
    fprintf('best model order (BIC) = %d\n',moBIC);
end

switch opt_gra.select_crit
    case 'AIC'
        morder = moAIC;
    case 'BIC'
        morder = moBIC;
    case 'actual'
        morder = opt_gra.morder;
end
if isnan(morder)
   sig = nan(nvars);
   pval = nan(nvars);
   return;
end
%% VAR complete model estimation

% Estimate VAR model of selected order from data.
[A,SIG] = tsdata_to_var(X,morder,opt_gra.regmode);
if isbad(A)
   sig = nan(nvars);
   pval = nan(nvars);
   return;
end
%% Autocovariance calculation

[G,info] = var_to_autocov(A,SIG,nobs);
if isbad(G) || info.error ~= 0
   sig = nan(nvars);
   pval = nan(nvars);
   return;
end

%% Granger causality calculation: time domain 
F = autocov_to_pwcgc(G);
F_prime = F;
for i = 1:nvars
    F_prime(i,i) = 1;
end
if isbad(F_prime) || size(F,1) == 1
   sig = nan(nvars);
   pval = nan(nvars);
   return;
end

pval = mvgc_pval(F,morder,nobs,1,1,1,nvars-2,opt_gra.tstat); % take careful note of arguments!
sig  = significance(pval,alpha,opt_gra.mhtc);

% Plot p-values and significance.
if opt_gra.visual == 1
    figure();
    subplot(1,2,1);
    plot_pw(pval);
    title('p-values');
    subplot(1,2,2);
    plot_pw(sig);
    title(['Significant at p = ' num2str(alpha)])
end
