function x = input_generation(opt_input)
% INPUT_GENERATION generates a vector of values representing the exogenous
%   input of a process (monodimensional)
%   Input:
%       opt_input.type: type of the process generating inputs
%           'cosine'        = cosine function
%               requires opt_input.per (period of the cosine function)
%               requires opt_input.noise_std (standard deviation of the
%                    introduced noise)
%           'random_walk'   = pseudo random walk
%               requires opt_sig.input.a (multiplicative coefficient for 
%                   the previous element of the walk)
%               requires opt_input.noise_std (standard deviation of the
%                    sampling gaussian distribution)
%           'uniform'       = uniformly random process
%       opt_input.hor: lenght of the signal to be generated
%
%   Output:
%       x: vector containing the values of the exogenous part of a process

switch opt_input.type
    case 'cosine'
        x = cos( 2 * pi * (1:opt_input.hor) / opt_input.per) ...
            + randn(1,opt_input.hor)*opt_input.noise_std;
    case 'random_walk'
        x = zeros(1,opt_input.hor);
        x0 = randn(1);
        x(1) = opt_input.a * x0 + randn() * opt_input.noise_std;
        for i = 2:opt_input.hor
            x(i) = opt_input.a * x(i-1) + randn() * opt_input.noise_std;
        end
    case 'uniform'
        x = (2 * rand(1,opt_input.hor) - 1);
end
