function topology = determine_topology(opt_top)
% DETERMINE_TOPOLOGY generates a random topology (dag) for a sensor network with
% fixed amount of nodes and edges
%   Input:
%       opt_top.n_nodes: number of sensors in the sensor network
%       opt_top.conn_perc: percentage of edges to be created in the
%           topology [0,0.5]
%       opt_top.verbose: flag for the visualization of the obtained graph
%
%   Output:
%       topology: adjecency matrix specifying existing relationships among 
%           sensors

max_edges = floor(opt_top.n_nodes * (opt_top.n_nodes - 1 ) * opt_top.conn_perc);
assert(opt_top.conn_perc <= 0.5);
contr = 0;

while contr == 0
    topology = sparse([],[],true,opt_top.n_nodes,opt_top.n_nodes);
    while nnz(topology) < max_edges
        %Add edge only if the graph is still a dag with the inserted edge
        edge = randsample(opt_top.n_nodes^2,1);
        topology(edge) = true;
        topology(edge) = graphisdag(topology);
    end
    
    if max_edges >= opt_top.n_nodes
        n_comp = graphconncomp(topology,'Weak',true);
        if n_comp == 1
            contr = 1;
        end
    else
        contr = 1;
    end
end

if opt_top.verbose == 1
    view(biograph(topology))
end
