function [data full_topology direct_topology opt_net_sig] = generate_net(sig_type,opt_top)
% GENERATE_NET generate the datastreams of a sensor network, whose topology
% is randomly generated according to specific in OPT_TOP and the signal
% generation is specified in SIG_TYPE
%   Input:
%       sig_type: structure specifying the data generation
%           (see GENERATE_OPTS for details)
%       opt_top: structure specifying the dependency graph generation 
%           (see DETERMINE_TOPOLOGY for details)
%
%   Output:
%       data: generated datastreams
%       full_topology: adjacency matrix for all possible reacheable sensors
%       direct_topology: adjacency matrix for direct connections
%       opt_net_sig: options for each one of the datastream generated

sig_type.n_input = 1;
[opt_input, opt_output, ~] = generate_opts(sig_type,[]);
data = zeros(opt_top.n_nodes,opt_output.hor);

first = 0;

while first == 0 || sum(isnan(data(:))) > 0 || sum(isinf(data(:))) > 0

    topology = determine_topology(opt_top);
    ordered_nodes = graphtopoorder(topology);
    matr_topology = full(topology);
    opt_net_sig = [];
    
    first = 1;
    for i = ordered_nodes
        if sum(matr_topology(:,i)) == 0
            data(i,:) = input_generation(opt_input);
        else
            input_ind = matr_topology(:,i) ~= 0;
            input = data(input_ind,:);
            sig_type.n_input = sum(input_ind);
            [opt_input opt_output ~] = generate_opts(sig_type,[]);
            opt_net_sig.opt_output(i) = opt_output;
            data(i,:) = signal_generation(input,opt_output);
        end
    end
end

opt_net_sig.hor = opt_output.hor;
opt_net_sig.n_train = opt_output.n_train;
data = data';

%Consider indirect connections
full_topology = zeros(opt_top.n_nodes);
for i = 1:opt_top.n_nodes
    reacheable_nodes = graphtraverse(sparse(topology), i);
    reacheable_nodes = setdiff(reacheable_nodes,i);
    full_topology(i,reacheable_nodes) = 1;
end
direct_topology = full(topology);
