function opt_est = arx_order_estimation(data,opt_data,opt_est)

assert(opt_est.train_est_perc > 0 && opt_est.train_est_perc <= 1);

if strcmp(opt_est.val,'AIC') || strcmp(opt_est.val,'BIC') || strcmp(opt_est.val,'FPE')
    opt_est.train_est_perc = 1;
end
    
x = data(:,opt_data.input);
y = data(:,opt_data.output);

%Training and validation set extraction
n_train_sam = floor(opt_data.hor * opt_est.train_est_perc);
y_train = y(1:n_train_sam);
y_valid = y(n_train_sam+1:end);
x_train = x(1:n_train_sam,:);
x_valid = x(n_train_sam+1:end,:);
n_input = size(x_train,2);

datax_train = iddata(y_train,x_train);
datax_valid = iddata(y_valid,x_valid);

%ARX
err_valid = zeros(opt_est.max_ord_expl,opt_est.max_ord_expl);
for a_ord = 1:opt_est.max_ord_expl
    for b_ord = 1:opt_est.max_ord_expl
        arx_mod = arx(datax_train,'na',a_ord,'nb',repmat(b_ord,1,n_input),'nk',ones(1,n_input));
        if strcmp(opt_est.val,'AIC')
            err_valid(a_ord,b_ord) = log(arx_mod.Report.Fit.MSE) + 2 * (a_ord + n_input * b_ord) / n_train_sam;
        elseif strcmp(opt_est.val,'BIC')
            err_valid(a_ord,b_ord) =  log(arx_mod.Report.Fit.MSE) + (a_ord + n_input * b_ord) * log(n_train_sam);
        elseif strcmp(opt_est.val,'FPE')
            err_valid(a_ord,b_ord) =  log(arx_mod.Report.Fit.MSE) * (1 - 2 * n_train_sam / (a_ord + n_input * b_ord));
        elseif strcmp(opt_est.val,'MSE')
            res = resid(arx_mod,datax_valid);
            err_valid(a_ord,b_ord) = sum(res.OutputData.^2) / n_train_sam;
        else
            fprintf('%s\n','Nonexisting option for order estimation');
            return;
        end
    end
end
mini = find(err_valid == min(min(err_valid(2:end,:))));
[opt_est.aord opt_est.bord] = ind2sub(size(err_valid),mini(1));
if opt_est.verbose == 1
    fprintf('%s%i%s%i%s\n','Chosen orders: ARX(',opt_est.aord,',',opt_est.bord,')');
end
